// Standard includes
#include <string.h>
#include <stdint.h>


// Driverlib includes
#include "hw_types.h"
#include "hw_memmap.h"
#include "hw_common_reg.h"
#include "hw_ints.h"
#include "spi.h"
#include "rom.h"
#include "rom_map.h"
#include "utils.h"
#include "prcm.h"
#include "uart.h"
#include "interrupt.h"
//for CRC
#include "crc.h"
#define CCM0_BASE           DTHE_BASE
// Common interface includes
#include "uart_if.h"

//for DMA
#include "udma.h"
//#include "udma_if.h"
#include "hw_mcspi.h"
#include "pin.h"
#define MAX_NUM_CH              64  //32*2 entries
#define CTL_TBL_SIZE            64  //32*2 entrie
typedef void (*tAppCallbackHndl)(void);

#define SPI_IF_BIT_RATE  800000
#define TR_BUFF_SIZE     16

static uint32_t interrupt_count;
static uint8_t tx_buffer[TR_BUFF_SIZE];
static uint8_t rx_buffer[TR_BUFF_SIZE];

/* used for DMA */
tDMAControlTable dma_control_table[CTL_TBL_SIZE] __attribute__(( aligned(1024)));
tAppCallbackHndl dma_callback_handler[MAX_NUM_CH];	


void
dma_error_handler(void)
{
    MAP_uDMAIntClear(MAP_uDMAIntStatus());
	Message("DMA Error\n\r");
}

void interrupt_handler()
{
	Message("int\n\r");
	MAP_SPIIntStatus(GSPI_BASE,true);
	MAP_SPIIntClear(GSPI_BASE,SPI_INT_DMARX|SPI_INT_DMATX);
	interrupt_count++;
}

void display_buffer(uint8_t* buffer, int len)
{
	if(len > 16)
	{
		Message("Buffer is too big, printing only the first 16 elements!\n\r");
		len = 16;
	}
	Report("Buffer:\n\r");
	for(int i = 0;i < len;i++)
	{
		Report("/%x",buffer[i]);
	}
	Report("/\n\r");
	Report("Last: %d\n\r",buffer[len-1]);

}
int main()
{
	interrupt_count = 0;

	uint32_t output_count = 0;
	uint32_t sum = 0;
	
	//-- init()

    MAP_IntMasterEnable();
    MAP_IntEnable(FAULT_SYSTICK);
    PRCMCC3200MCUInit();

   // PinMuxConfig();
	MAP_PRCMPeripheralClkEnable(PRCM_UARTA0, PRCM_RUN_MODE_CLK);
    MAP_PRCMPeripheralClkEnable(PRCM_GSPI, PRCM_RUN_MODE_CLK);
    MAP_PinTypeUART(PIN_55, PIN_MODE_3);
    MAP_PinTypeUART(PIN_57, PIN_MODE_3);
    MAP_PinTypeSPI(PIN_05, PIN_MODE_7);
    MAP_PinTypeSPI(PIN_06, PIN_MODE_7);
    MAP_PinTypeSPI(PIN_07, PIN_MODE_7);
    MAP_PinTypeSPI(PIN_08, PIN_MODE_7);

    //MAP_PRCMPeripheralClkEnable(PRCM_GSPI,PRCM_RUN_MODE_CLK);
    InitTerm();
    ClearTerm();
    Message("\n\n\n\r");
    Message("\t\t   ********************************************\n\r");
    Message("\t\t        CC3200 SPI Demo Application!  \n\r");
    Report("\t\t        Birate:  %d  \n\r", SPI_IF_BIT_RATE);
    Report("\t\t        Buffsize:%d  \n\r", TR_BUFF_SIZE  );
    Message("\t\t   ********************************************\n\r");
    Message("\n\n\n\r");
    MAP_PRCMPeripheralReset(PRCM_GSPI);
	// for CRC
	MAP_PRCMPeripheralClkEnable(PRCM_DTHE, PRCM_RUN_MODE_CLK);
	//--
	MAP_SPIReset(GSPI_BASE);
	// DMA
	// setup
	//-----------
	//This block is UDMAInit()
	
	MAP_PRCMPeripheralClkEnable(PRCM_UDMA,PRCM_RUN_MODE_CLK);
	MAP_PRCMPeripheralReset(PRCM_UDMA);
	MAP_uDMAIntRegister(UDMA_INT_ERR, dma_error_handler);
	MAP_uDMAEnable();
	memset(dma_control_table,0,sizeof(tDMAControlTable)*CTL_TBL_SIZE);
    MAP_uDMAControlBaseSet(dma_control_table);	 
    for(uint32_t i = 0; i < MAX_NUM_CH; i++)
    {
        dma_callback_handler[i] = NULL;
    }
	//UDMAInit();
	//-----------
    MAP_SPIConfigSetExpClk(GSPI_BASE,MAP_PRCMPeripheralClockGet(PRCM_GSPI),
                     SPI_IF_BIT_RATE,SPI_MODE_SLAVE,SPI_SUB_MODE_0,
                     (SPI_HW_CTRL_CS |
                     SPI_4PIN_MODE |
                     SPI_TURBO_OFF |
                     SPI_CS_ACTIVEHIGH |
                     SPI_WL_8));	
	MAP_SPIIntRegister(GSPI_BASE,interrupt_handler);
	/*
	//FIFO
	MAP_uDMAChannelAssign( UDMA_CH30_GSPI_RX);
	MAP_uDMAChannelAssign( UDMA_CH31_GSPI_TX );
	MAP_uDMAChannelAttributeDisable(UDMA_CH30_GSPI_RX , UDMA_ATTR_ALTSELECT );
    MAP_uDMAChannelAttributeDisable( UDMA_CH31_GSPI_TX, UDMA_ATTR_ALTSELECT );
	MAP_SPIFIFOEnable( GSPI_BASE, SPI_RX_FIFO );
    MAP_SPIFIFOEnable( GSPI_BASE, SPI_TX_FIFO ); 
	MAP_SPIDmaEnable( GSPI_BASE, SPI_RX_DMA );
    MAP_SPIDmaEnable( GSPI_BASE, SPI_TX_DMA );
	MAP_SPIFIFOLevelSet( GSPI_BASE, 1, 1 );
	MAP_IntRegister( INT_GSPI, (void(*)(void))interrupt_handler );
	MAP_IntPrioritySet( INT_GSPI, INT_PRIORITY_LVL_0 );
	MAP_IntEnable( INT_GSPI );
	MAP_SPIIntEnable( GSPI_BASE, SPI_INT_EOW );
	MAP_SPIWordCountSet( GSPI_BASE, TR_BUFF_SIZE );
	*/
	// same calls as SetupTransfer
	// channel 
	// RX
	MAP_uDMAChannelControlSet(UDMA_CH30_GSPI_RX,UDMA_SIZE_8 | 
												UDMA_SRC_INC_NONE | 
												UDMA_DST_INC_8 |
												UDMA_ARB_1);
	MAP_uDMAChannelAttributeEnable(UDMA_CH30_GSPI_RX,UDMA_ATTR_USEBURST);
	MAP_uDMAChannelTransferSet(UDMA_CH30_GSPI_RX,UDMA_MODE_BASIC,
								(void *)(GSPI_BASE + MCSPI_O_RX0),
								(void *) &rx_buffer,
								TR_BUFF_SIZE);
	MAP_uDMAChannelEnable(UDMA_CH30_GSPI_RX);
	// TX
	MAP_uDMAChannelControlSet(UDMA_CH31_GSPI_TX,UDMA_SIZE_8 | 
												UDMA_SRC_INC_8 | 
												UDMA_DST_INC_NONE |
												UDMA_ARB_1);
	MAP_uDMAChannelAttributeEnable(UDMA_CH31_GSPI_TX,UDMA_ATTR_USEBURST);
	MAP_uDMAChannelTransferSet(UDMA_CH31_GSPI_TX,UDMA_MODE_BASIC,
								(void *)&tx_buffer,
								(void *)(GSPI_BASE + MCSPI_O_TX0),
								TR_BUFF_SIZE );
	MAP_uDMAChannelEnable(UDMA_CH31_GSPI_TX);

	//--------------
	
	/*
  SetupTransfer(UDMA_CH30_GSPI_RX,UDMA_MODE_BASIC,TR_BUFF_SIZE,
                UDMA_SIZE_8,UDMA_ARB_1,
                (void *)(GSPI_BASE + MCSPI_O_RX0),UDMA_SRC_INC_NONE,
                &rx_buffer,UDMA_DST_INC_8);

  SetupTransfer(UDMA_CH31_GSPI_TX,UDMA_MODE_BASIC,TR_BUFF_SIZE,
                UDMA_SIZE_8,UDMA_ARB_1,
                &tx_buffer,UDMA_SRC_INC_8,(void *)(GSPI_BASE + MCSPI_O_TX0),
                UDMA_DST_INC_NONE);
*/
	SPIDmaEnable(GSPI_BASE,SPI_RX_DMA|SPI_TX_DMA);
	MAP_SPIIntEnable(GSPI_BASE,SPI_INT_DMATX | SPI_INT_DMARX );
	MAP_SPIEnable(GSPI_BASE);

	// initialize buffers with some known value
	memset(tx_buffer,0x55,sizeof(tx_buffer));
	memset(rx_buffer,0x22,sizeof(rx_buffer));
	for(int i = 0; i < TR_BUFF_SIZE; i++)
	{
		tx_buffer[i] = 0x55 + i;
	}



    Message("Enabled SPI Interface in Slave Mode!\n\r");

	Message("Starting while\n\r");
    while(1)
    {
	//	memcpy(tx_buffer,rx_buffer,TR_BUFF_SIZE);
		sum = 0;
//		if(output_count < interrupt_count)
		{
			for(int i = 0; i < TR_BUFF_SIZE; i++)
			{
				sum += rx_buffer[i];
			}
			Report("The sum in the Rx buffer is: %d\n\r",sum);
			//Report("Checksum Rx buffer is: 0x%02x\n\r",crc(rx_buffer));
			Message("TX-");
			display_buffer(tx_buffer,TR_BUFF_SIZE);
			Message("RX-");
			display_buffer(rx_buffer,TR_BUFF_SIZE);
			Report("interrupt: %d\n\r",interrupt_count);
			output_count++;
		}
		MAP_UtilsDelay(800000*20);
    }
	return 0;
}

